﻿#region Header
// ---------------------
// Silverlight 3 Samples
// ---------------------
//
// Project:         Animatable MediaElement.
// Current Version: 1.0 / Sept 2009
//
// Copyright:       (c) 2009 Olivier Dahan
//                  odahan@e-naxos.com
//                  DotBlog: www.e-naxos.com/blog
// 
// Revision History:
// -----------------
// Sept 2009 - V 1.0 
#endregion


#region Disclaimer
// I'm the author of this source code. 
// I've placed this source code in the public domain. 
// You can use, modify, and distribute the source code and executable programs based on the source code. 
// However, note the following: 
// DISCLAIMER OF WARRANTY 
// This source code is provided "as is" and without warranties as to performance or merchantability. 
// The author and/or distributors of this source code may have made statements about this source code. 
// Any such statements do not constitute warranties and shall not be relied on by the user in deciding 
// whether to use this source code. 
// This source code is provided without any express or implied warranties whatsoever. 
// Because of the diversity of conditions and hardware under which this source code may be 
// used, no warranty of fitness for a particular purpose is offered. The user is advised 
// to test the source code thoroughly before relying on it. The user must assume the 
// entire risk of using the source code.  
#endregion


#region Usings
using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media; 
#endregion

namespace SyncSound
{
    /// <summary>
    /// A <see cref="MediaElement"/> that is animatable through a dependency property.
    /// A sound or music can then be synchronized with other animated elements.
    /// The AnimatablePlay is the main property, the others are just relays to
    /// underlying <c>MediaElement</c> (Source, Volume...). All methods are also relays
    /// to the internal <c>MediaElement</c> (Stop, Play). PlayFromStart is a new
    /// method sending a Stop then a Play as this sequence is often useful.
    /// </summary>
    public partial class SynchedSoundPlayer : UserControl
    {
        #region Ctor and internal stuff
        public SynchedSoundPlayer()
        {
            // Required to initialize variables
            InitializeComponent();
            Loaded += synchedSoundPlayer_Loaded;
        }

        private void synchedSoundPlayer_Loaded(object sender, RoutedEventArgs e)
        {
            internalMP.AutoPlay = false;
            internalMP.MediaFailed += internalMP_MediaFailed;
        }


        private void internalMP_MediaFailed(object sender, ExceptionRoutedEventArgs e)
        {
            if (MediaFailed != null) MediaFailed(this, e);
        }
        #endregion

        #region Public Event
        /// <summary>
        /// Occurs when internal <see cref="MediaElement"/> fires <c>MediaFailed</c> event.
        /// </summary>
        public event EventHandler<ExceptionRoutedEventArgs> MediaFailed;
        #endregion

        #region Source property
        [Category("Media")]
        public Uri Source
        {
            get { return (Uri)GetValue(SourceProperty); }
            set { SetValue(SourceProperty, value); }
        }

        public static readonly DependencyProperty SourceProperty =
            DependencyProperty.Register("Source", typeof(Uri),
            typeof(SynchedSoundPlayer),
            new PropertyMetadata(null, SourceChanged));

        private static void SourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((SynchedSoundPlayer)d).internalMP.Source = (Uri)e.NewValue;
        }
        #endregion

        #region Volume property
        [Category("Media")]
        public double Volume
        {
            get { return (double)GetValue(VolumeProperty); }
            set { SetValue(VolumeProperty, value); }
        }

        public static readonly DependencyProperty VolumeProperty =
            DependencyProperty.Register("Volume", typeof(double),
            typeof(SynchedSoundPlayer), new PropertyMetadata(0.5, VolumeChanged));

        private static void VolumeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((SynchedSoundPlayer)d).internalMP.Volume = (double)e.NewValue;
        }
        #endregion

        #region Balance property
        [Category("Media")]
        public double Balance
        {
            get { return (double)GetValue(BalanceProperty); }
            set { SetValue(BalanceProperty, value); }
        }

        public static readonly DependencyProperty BalanceProperty =
            DependencyProperty.Register("Balance", typeof(double),
            typeof(SynchedSoundPlayer), new PropertyMetadata(0.0, BalanceChanged));

        private static void BalanceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((SynchedSoundPlayer)d).internalMP.Balance = (double)e.NewValue;
        }
        #endregion

        #region Position property
        [Category("Media")]
        public TimeSpan Position
        {
            get { return (TimeSpan)GetValue(PositionProperty); }
            set { SetValue(PositionProperty, value); }
        }

        public static readonly DependencyProperty PositionProperty =
            DependencyProperty.Register("Position", typeof(TimeSpan),
            typeof(SynchedSoundPlayer), new PropertyMetadata(TimeSpan.FromTicks(0), PositionChanged));

        private static void PositionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((SynchedSoundPlayer)d).internalMP.Position = (TimeSpan)e.NewValue;
        }
        #endregion

        #region AnimatablePlay property
        /// <summary>
        /// Gets or sets the animatable play.
        /// Value = 0 means <see cref="Stop"/>
        /// Value sup.to 0 means <see cref="PlayFromStart"/>
        /// Value inf.to 0 means <see cref="Play"/> / <see cref="Pause"/>
        /// </summary>
        /// <value>The animatable play.</value>
        [Category("Media")]
        public double AnimatablePlay
        {
            get { return (double)GetValue(AnimatablePlayProperty); }
            set { SetValue(AnimatablePlayProperty, value); }
        }

        public static readonly DependencyProperty AnimatablePlayProperty =
            DependencyProperty.Register("AnimatablePlay", typeof(double),
            typeof(SynchedSoundPlayer), new PropertyMetadata(0.0d, AnimatablePlayChanged));

        private static void AnimatablePlayChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var mp = ((SynchedSoundPlayer)d);
            if ((double)e.NewValue > 0d) mp.PlayFromStart();
            if ((double)e.NewValue == 0d) mp.Stop();
            if ((double)e.NewValue < 0d)
            {
                if (mp.internalMP.CurrentState == MediaElementState.Playing) mp.Pause();
                else mp.Play();
            }
        }

        #endregion

        #region Public methods
        public void Play()
        { internalMP.Play(); }

        public void PlayFromStart()
        { internalMP.Stop(); internalMP.Play(); }

        public void Stop()
        { internalMP.Stop(); }

        public void Pause()
        { internalMP.Pause(); }
        #endregion

    }
}